@@ -5,12 +5,28 @@ from typing import Callable, Iterator, List, Optional, Tuple
 
 from fastapi.routing import APIRoute
 from fastapi.params import Depends
-from starlette.responses import JSONResponse
+from starlette.responses import JSONResponse, Response
+from starlette.requests import Request
 
 from apish.base import Problem
 from apish.utils import get_return_type
 
 
+class CustomRoute(APIRoute):
+    def get_route_handler(self) -> Callable:
+        original_route_handler = super().get_route_handler()
+
+        async def custom_route_handler(request: Request) -> Response:
+            try:
+                return await original_route_handler(request)
+            except Exception as exc:
+                body = await request.body()
+                request.state.body = body
+                raise exc
+
+        return custom_route_handler
+
+
 class Verb(Enum):
     GET = auto()
     PUT = auto()
@@ -107,4 +123,4 @@ class Resource(RouteGenerator):
         if response_model is not None:
             kwargs["response_model"] = response_model
 
-        return APIRoute(**kwargs)
+        return CustomRoute(**kwargs)
